package com.acme.test;

import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import com.acme.MyAdapter;
import com.sun.grizzly.http.SelectorThread;
import com.sun.grizzly.standalone.StaticStreamAlgorithm;
import com.sun.grizzly.tcp.Adapter;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import org.apache.commons.httpclient.HttpConnectionManager;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

/**
 *
 */
public class TestSuspend
{
    int             port = 35000;
    SelectorThread  selector;
    Adapter         adapter;
    String          url;
    AtomicInteger   succes;
    HttpConnectionManager   manager;

    @Before
    public void before()
        throws Exception
    {
        // Create the adapter
        adapter = new MyAdapter( true );

        // Create the selector
        selector = new SelectorThread();
        selector.setPort( port );
        selector.setEnableAsyncExecution( false );
        selector.setAdapter( adapter );
        selector.setAlgorithmClassName( StaticStreamAlgorithm.class.getName() );
        selector.setDisplayConfiguration( true );
        selector.listen();

        // Others
        succes = new AtomicInteger(0);
        url = "http://localhost:"+port+"/ping";
        manager = new MultiThreadedHttpConnectionManager();
        
    }

    @After
    public void after()
    {
        selector.stopEndpoint();
    }

    
    @Test
    public void testManyRequestsInParallel()
        throws Exception
    {
        ThreadPoolExecutor  executor;
        int                 nbthread ;
        int                 count;
        Caller              caller;

        nbthread = 180;
        count = 10000;
        manager.getParams().setDefaultMaxConnectionsPerHost( nbthread*2 );
        manager.getParams().setMaxTotalConnections( nbthread*2 );
        executor = new ThreadPoolExecutor( nbthread, nbthread, 5*60, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>() );
        for( int i=0; i<nbthread; i++ )
        {
            caller = new Caller( url, count, succes, manager );
            executor.execute( caller );
        }

        try
		{
			executor.shutdown();
			executor.awaitTermination( 15, TimeUnit.MINUTES ); // wait for all to finish
		}
		catch (InterruptedException e)
		{
			fail( "Interrupted during requesters execution." );
		}

        System.out.println( "\n** Success: "+succes.get()+" / "+(nbthread*count)+"\n" );
        
        assertEquals( nbthread*count, succes.get() );
    }

}
